(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 11.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[     13819,        366]
NotebookOptionsPosition[     13087,        340]
NotebookOutlinePosition[     13434,        355]
CellTagsIndexPosition[     13391,        352]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["Convert amp to decibels", "Text",
 CellChangeTimes->{{3.6910737905323906`*^9, 3.6910738172184253`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"a2db", "[", "x_", "]"}], ":=", 
  RowBox[{"20", " ", 
   RowBox[{"Log", "[", 
    RowBox[{"10", ",", 
     RowBox[{"Abs", "[", "x", "]"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.691073915955385*^9, 3.6910739193929276`*^9}}],

Cell[" Convert  decibels to amp", "Text",
 CellChangeTimes->{{3.6910738874768047`*^9, 3.691073909921235*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"db2a", "[", "x_", "]"}], ":=", 
  SuperscriptBox["10", 
   RowBox[{"(", 
    RowBox[{"x", "/", "20"}], ")"}]]}]], "Input",
 CellChangeTimes->{{3.6910739270335855`*^9, 3.691073957230051*^9}}],

Cell["\<\
 Returns a (real) time domain minimum phase FIR filter kernel from (real) \
input time domain FIR filter kernel \
\>", "Text",
 CellChangeTimes->{3.6908692301175385`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ReMps", "[", "s_", "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Re", "[", 
    RowBox[{"InverseFourier", "[", 
     RowBox[{"Mps", "[", 
      RowBox[{"Fourier", "[", 
       RowBox[{"Re", "[", "s", "]"}], "]"}], "]"}], "]"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.690868273529825*^9, 3.6908683172263246`*^9}}],

Cell["\<\
Returns a minimum phase spectrum from the complex spectrum s \
\>", "Text",
 CellChangeTimes->{3.6908692478475533`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Mps", "[", "s_", "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Exp", "[", "\[IndentingNewLine]", 
    RowBox[{"Fourier", "[", "\[IndentingNewLine]", 
     RowBox[{"FoldBack", "[", 
      RowBox[{"InverseFourier", "[", "\[IndentingNewLine]", 
       RowBox[{"Log", "[", 
        RowBox[{"Clipdb", "[", 
         RowBox[{"s", ",", 
          RowBox[{"-", "300"}]}], "]"}], "]"}], "]"}], "]"}], "]"}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6908683376264915`*^9, 3.69086839651886*^9}, 
   3.691074009464305*^9}],

Cell["\<\
Clip the magnitude of s at its maximum + cutoff in dB. Example Clipdb[s, -N] \
returns s with the minimum magnitude of s not more than NdB below its maximum \
magnitude \
\>", "Text",
 CellChangeTimes->{3.6908692604972763`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clipdb", "[", 
   RowBox[{"s_", ",", "cutoff_"}], "]"}], ":=", "\[IndentingNewLine]", 
  RowBox[{"Block", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"as", ",", "thresh", ",", "i"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"as", "=", 
      RowBox[{"Abs", "[", "s", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"thresh", "=", 
      RowBox[{
       RowBox[{"Max", "[", "as", "]"}], "*", 
       SuperscriptBox["10", 
        RowBox[{"(", 
         RowBox[{"cutoff", "/", "20"}], ")"}]]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Table", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"as", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "<", "thresh"}], ",", "thresh", 
         ",", 
         RowBox[{"s", "[", 
          RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", 
         RowBox[{"Length", "[", "s", "]"}]}], "}"}]}], "\[IndentingNewLine]", 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.690868429726759*^9, 3.690868601915608*^9}, {
   3.690868674560763*^9, 3.6908687163751545`*^9}, 3.6908688090304537`*^9, {
   3.6908702199271526`*^9, 3.6908702289106665`*^9}, {3.690870376417104*^9, 
   3.690870438464652*^9}, 3.6908705316599827`*^9, {3.690870577218589*^9, 
   3.690870598056781*^9}, {3.6908706436613894`*^9, 3.6908706450354676`*^9}, {
   3.6908706975324707`*^9, 3.690870742615049*^9}, {3.690870779240144*^9, 
   3.690870784103422*^9}, {3.6908708724774766`*^9, 3.6908709316888633`*^9}, {
   3.690871053978858*^9, 3.690871054560891*^9}, {3.6908710865277195`*^9, 
   3.6908710898509097`*^9}, {3.6908711509494042`*^9, 
   3.6908712303449454`*^9}, {3.690871288237257*^9, 3.6908712902703733`*^9}, {
   3.691073556173089*^9, 3.6910735564387836`*^9}}],

Cell["\<\
This \[OpenCurlyDoubleQuote]time-aliases\[CloseCurlyDoubleQuote] the \
non-causal part of a function onto its causal part. When applied to the \
inverse fourier transform of a log-spectrum, it converts non-minimum-phase \
zeros to minimum-phase zeros.
   Fold the left wing of fft vector onto the right wing. \
\>", "Text",
 CellChangeTimes->{3.6908692796473722`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FoldBack", "[", "rin_", "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"r", "=", "rin"}], ",", 
       RowBox[{"n", "=", 
        RowBox[{"Length", "[", "rin", "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"m", "=", 
        RowBox[{
         RowBox[{"Depth", "[", "rin", "]"}], "-", "1"}]}], ",", 
       RowBox[{"flipped", "=", "0"}], ",", 
       RowBox[{"rw", "=", "0"}], ",", "\[IndentingNewLine]", "rf"}], "}"}], 
     ",", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"m", " ", "n"}], " ", "\[NotEqual]", " ", 
         RowBox[{"m", "+", "n", "-", "1"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
        "Return", "[", "\[IndentingNewLine]", 
         "\"\<Error: input must be a vector\>\"", "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"m", ">", "n"}], ",", 
        RowBox[{
         RowBox[{"n", "=", "m"}], ";", 
         RowBox[{"r", "=", 
          RowBox[{"Transpose", "[", "r", "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"flipped", "=", "1"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"n", "<", "3"}], ",", 
        RowBox[{"Return", "[", "r", "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Mod", "[", 
          RowBox[{"n", ",", "2"}], "]"}], "\[Equal]", "1"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"nt", "=", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"n", "+", "1"}], ")"}], "/", "2"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"rw", "=", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"r", "[", 
             RowBox[{"[", "1", "]"}], "]"}], ",", 
            RowBox[{
             RowBox[{"Take", "[", 
              RowBox[{"r", ",", 
               RowBox[{"{", 
                RowBox[{"2", ",", "nt"}], "}"}]}], "]"}], "+", 
             "\[IndentingNewLine]", 
             RowBox[{"Conjugate", "[", "\[IndentingNewLine]", 
              RowBox[{"Reverse", "[", 
               RowBox[{"Take", "[", 
                RowBox[{"r", ",", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"nt", "+", "1"}], ",", "n"}], "}"}]}], "]"}], 
               "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
            RowBox[{"Table", "[", 
             RowBox[{"0", ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", "1", ",", 
                RowBox[{"n", "-", "nt", "-", "1"}]}], "}"}]}], "]"}]}], 
           "}"}]}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"nt", "=", 
          RowBox[{"n", "/", "2"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"rf", "=", 
          RowBox[{"Flatten", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Take", "[", 
               RowBox[{"r", ",", 
                RowBox[{"{", 
                 RowBox[{"2", ",", "nt"}], "}"}]}], "]"}], ",", "0"}], "}"}], 
            ",", "1"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"rf", "=", "\[IndentingNewLine]", 
          RowBox[{"rf", "+", 
           RowBox[{"Conjugate", "[", "\[IndentingNewLine]", 
            RowBox[{"Reverse", "[", 
             RowBox[{"Take", "[", 
              RowBox[{"r", ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"nt", "+", "1"}], ",", "n"}], "}"}]}], "]"}], "]"}], 
            "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"rw", "=", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"r", "[", 
             RowBox[{"[", "1", "]"}], "]"}], ",", "rf", ",", 
            "\[IndentingNewLine]", 
            RowBox[{"Table", "[", 
             RowBox[{"0", ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", "1", ",", 
                RowBox[{"n", "-", "nt", "-", "1"}]}], "}"}]}], "]"}]}], 
           "}"}]}]}]}], "\[IndentingNewLine]", "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"rw", "=", 
       RowBox[{"Flatten", "[", 
        RowBox[{"rw", ",", "1"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"flipped", "\[Equal]", "1"}], ",", 
        RowBox[{"rw", "=", 
         RowBox[{"Transpose", "[", "rw", "]"}]}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Return", "[", "rw", "]"}], ";"}]}], "\[IndentingNewLine]", 
    "]"}]}], "\[IndentingNewLine]"}]], "Input",
 CellChangeTimes->{{3.6908693062648945`*^9, 3.690869784524249*^9}}],

Cell["\<\
Application to generating a minimum phase sinc
  The ideal and array sinc function are generated below. A minimum phase \
version of the sinc function is generated using the Mps function. This is \
shown to have the same magnitude response and the original sinc function. The \
time domain FIR filter kernels are then plotted for each. \
\>", "Text",
 CellChangeTimes->{3.690871316863894*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"tabsize", "=", 
   RowBox[{"65536", "*", "8"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"sincfir", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Sinc", "[", 
      RowBox[{"1000.", " ", "2", " ", "Pi", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"x", "-", 
          RowBox[{"tabsize", "/", "2"}]}], ")"}], " ", "/", "44100"}]}], 
      "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "tabsize"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"mpsincfir", " ", "=", 
   RowBox[{"ReMps", "[", "sincfir", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"sincfir", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "0.5"}], ",", "1"}], "}"}]}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"mpsincfir", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "0.5"}], ",", "1"}], "}"}]}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Export", "[", 
  RowBox[{"\"\<sincfir.csv\>\"", ",", "sincfir"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Export", "[", 
  RowBox[{"\"\<mpsincfir.csv\>\"", ",", "mpsincfir"}], 
  "]"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.690871329994645*^9, 3.690871745894433*^9}, {
   3.690871795866291*^9, 3.6908718029516964`*^9}, {3.6909868269884443`*^9, 
   3.690986828035331*^9}, {3.690986904973692*^9, 3.6909869445991364`*^9}, {
   3.6909869820682535`*^9, 3.6909869945371947`*^9}, {3.690987049772189*^9, 
   3.690987166101616*^9}, {3.690987206195751*^9, 3.6909872415711884`*^9}, {
   3.6909893900316725`*^9, 3.690989488201777*^9}, {3.6909895740309315`*^9, 
   3.690989583999792*^9}, {3.6909896773461046`*^9, 3.6909896929232054`*^9}, {
   3.690989798223486*^9, 3.690989801457898*^9}, {3.690989869036256*^9, 
   3.6909898827539477`*^9}, 3.690989927768448*^9, {3.690990020737773*^9, 
   3.6909900221458116`*^9}, {3.6910481132142963`*^9, 3.691048115667449*^9}, {
   3.69104815869913*^9, 3.691048170855567*^9}, {3.6910496109422226`*^9, 
   3.691049639224148*^9}, {3.6910737177207327`*^9, 3.6910737324851885`*^9}, {
   3.6910740552883205`*^9, 3.6910741857558517`*^9}, {3.691074221545548*^9, 
   3.6910742220454865`*^9}, {3.6910743979452963`*^9, 3.691074401914615*^9}, {
   3.6910744328052278`*^9, 3.6910744351021414`*^9}, {3.6910745911756887`*^9, 
   3.6910745930663385`*^9}, {3.691074683120332*^9, 3.691074684120397*^9}, {
   3.691074900011606*^9, 3.6910749007402053`*^9}, {3.691075096800047*^9, 
   3.691075106902728*^9}, 3.6910753391206894`*^9}]
},
WindowSize->{1902, 978},
WindowMargins->{{-9, Automatic}, {Automatic, 0}},
FrontEndVersion->"11.0 for Microsoft Windows (64-bit) (September 21, 2016)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[558, 20, 109, 1, 30, "Text"],
Cell[670, 23, 264, 7, 30, "Input"],
Cell[937, 32, 109, 1, 30, "Text"],
Cell[1049, 35, 225, 6, 30, "Input"],
Cell[1277, 43, 180, 4, 30, "Text"],
Cell[1460, 49, 374, 10, 50, "Input"],
Cell[1837, 61, 129, 3, 30, "Text"],
Cell[1969, 66, 595, 14, 126, "Input"],
Cell[2567, 82, 237, 5, 30, "Text"],
Cell[2807, 89, 1931, 43, 187, "Input"],
Cell[4741, 134, 377, 7, 49, "Text"],
Cell[5121, 143, 4814, 124, 582, "Input"],
Cell[9938, 269, 402, 7, 49, "Text"],
Cell[10343, 278, 2740, 60, 164, "Input"]
}
]
*)

